#!/bin/bash
# -*- mode:shell-script; coding:utf-8; -*-
#
# Created: <Thu Aug  3 14:05:20 2017>
# Last Updated: <2017-August-17 10:01:12>
#

scriptname=${0##*/}
orgname=""
envname=""
mapname=""
wait_time=4
mgmtserver=""
defaultmgmtserver="https://api.enterprise.apigee.com"
auditBase='/v1/audits/organizations'
org_cps_status=0
verbosity=1
netrccreds=0
credentials=""

usage() {
  local CMD=`basename $0`
  echo "$CMD: "
  echo "  Updates an environment-scoped KVM and checks audit log for a record of that updatde." 
  echo "  Uses the curl utility."
  echo "usage: "
  echo "  $CMD [options] "
  echo "options: "
  echo "  -m url    optional. the base url for the mgmt server. default: $defaultmgmtserver"
  echo "  -o org    required. the org to use."
  echo "  -e env    optional. the environment to use. default: ${envname}"
  echo "  -u creds  optional. http basic authn credentials for the API calls."
  echo "  -M map    required. KVM Map name."
  echo "  -n        optional. tells curl to use .netrc to retrieve credentials"
  echo "  -U        optional. tells the script to update a KVM entry."
  echo "  -D        optional. tells the script to delete a KVM entry."
  echo "  -A        optional. tells the script to add a KVM entry."
  echo
  echo "One of -U -D -A is required."
  echo
  exit 1
}

## function MYCURL
## Print the curl command, omitting sensitive parameters, then run it.
## There are side effects:
## 1. puts curl output into file named ${CURL_OUT}. If the CURL_OUT
##    env var is not set prior to calling this function, it is created
##    and the name of a tmp file in /tmp is placed there.
## 2. puts curl http_status into variable CURL_RC
MYCURL() {
    [ -z "${CURL_OUT}" ] && CURL_OUT=`mktemp /tmp/apigee-${scriptname}.curl.out.XXXXXX`
  [[ -f "${CURL_OUT}" ]] && rm ${CURL_OUT}
  [[ $verbosity -gt 0 ]] && echo "curl $@"

  # run the curl command
  CURL_RC=`curl $credentials -s -w "%{http_code}" -o "${CURL_OUT}" "$@"`
  [[ $verbosity -gt 0 ]] && echo "==> ${CURL_RC}"
}

choose_credentials() {
  local username password

  read -p "username for Edge org ${org} at ${mgmtserver} ? (blank to use .netrc): " username
  echo
  if [[ "$username" = "" ]] ; then  
    credentials="-n"
  else
    echo -n "Org Admin Password: "
    read -s password
    echo
    credentials="-u ${username}:${password}"
  fi
}

maybe_ask_password() {
  local password
  if [[ ${credentials} =~ ":" ]]; then
    credentials="-u ${credentials}"
  else
    echo -n "password for ${credentials}?: "
    read -s password
    echo
    credentials="-u ${credentials}:${password}"
  fi
}

check_org() {
  [[ $verbosity -gt 0 ]] && echo "checking org ${orgname}..."
  MYCURL -X GET  ${mgmtserver}/v1/o/${orgname}
  cat ${CURL_OUT} | grep -A 1 features.isCpsEnabled | grep value | grep -q true
  org_cps_status=$?
  if [[ ${org_cps_status} -ne 0 ]]; then
      printf "the org is not CPS enabled.\n"
  else
      printf "the org is CPS enabled.\n"
  fi
  [[ ${CURL_RC} -ne 200 ]]
}


CleanUp() {
  [[ -f ${CURL_OUT} ]] && rm -rf ${CURL_OUT}
}


kvm_entry_exists() {
    local keyname=$1
    MYCURL -X GET ${baseurl}/${mapname}/entries/${keyname}
    if [[ ${CURL_RC} -eq 200 ]]; then
        printf "The KVM entry exists.\n"
    else 
        printf "The KVM entry Does Not Exist.\n"
    fi
    [[ ${CURL_RC} -eq 200 ]]
}

insert_kvm_entry() {
    local payload
    local TIMESTAMP=$(date '+%Y%m%d-%H%M%S')
    local keyname="test-key-${TIMESTAMP}"
    local right_now_millis=$(date +%s000)
    if [[ ${org_cps_status} -eq 0 ]]; then
        # CPS enabled
        payload=$'{\n'
        payload+=$'  "name" : "'$keyname$'",\n'
        payload+=$'  "value" : "created at approximately '
        payload+="$right_now_millis"
        payload+=$'"\n'
        payload+=$'}'

        if kvm_entry_exists $keyname ; then
            # Update
            MYCURL -X POST ${baseurl}/${mapname}/entries/${keyname} -H content-type:application/json \
                   -d "${payload}"
        else
            # Create
            MYCURL -X POST ${baseurl}/${mapname}/entries -H content-type:application/json \
                   -d "${payload}"

        fi
        cat ${CURL_OUT}
    else
        # not CPS enabled
        payload=$'{\n'
        payload+=$'  "name" : "'$mapname$'",\n'
        payload+=$'  "entry" : [ { '
        payload+=$'  "name" : "'$keyname$'",\n'
        payload+=$'  "value" : "created at approximately '
        payload+="$right_now_millis"
        payload+=$'"\n'
        payload+=$'  } ]\n'
        payload+=$'}'

        MYCURL -X POST ${baseurl}/${mapname} -H content-type:application/json \
               -d "${payload}"
        
        cat ${CURL_OUT}
    fi
    printf "\n\n"
}


while getopts "hm:o:e:M:u:nUDA" opt; do
  case $opt in
    h) usage ;;
    m) mgmtserver=$OPTARG ;;
    o) orgname=$OPTARG ;;
    e) envname=$OPTARG ;;
    M) mapname=$OPTARG ;;
    u) credentials="-u $OPTARG" ;;
    n) netrccreds=1 ;;
    U) operation="update" ;;
    D) operation="delete" ;;
    A) operation="add" ;;
    *) echo "unknown arg" && usage ;;
  esac
done

echo
if [[ "X$mgmtserver" = "X" ]]; then
  mgmtserver="$defaultmgmtserver"
fi

if [[ "X$orgname" = "X" ]]; then
    printf "You must specify an org name (-o).\n\n"
    usage
    exit 1
fi

if [[ "X$mapname" = "X" ]]; then
    printf "You must specify an map name (-M).\n\n"
    usage
    exit 1
fi

if [[ "X$credentials" = "X" ]]; then
  if [[ ${netrccreds} -eq 1 ]]; then
    credentials='-n'
  else
    choose_credentials
  fi 
else
  maybe_ask_password
fi 

if check_org ; then
  printf "that org cannot be validated.\n"
  CleanUp
  exit 1
fi


baseurl=${mgmtserver}/v1/o/${orgname}/e/${envname}/keyvaluemaps

MYCURL -X GET ${baseurl}/${mapname}
if [[ ${CURL_RC} -ne 200 ]]; then
    printf "the map '%s' does not exist.\n" $mapname
    CleanUp
    exit 1
fi

printf "the map '%s' exists.\n" $mapname

insert_kvm_entry

printf "waiting just a bit..."
sleep $wait_time
printf "\n\n"

auditurl=${mgmtserver}${auditBase}/${orgname}
previous_midnight=$(date  -v0H -v0M -v0S +%s000)
next_midnight=$(date -v+1d -v0H -v0M -v0S +%s000)
#rightNow=$(date +%s000)
sixtySecondsAgo=$(($(date +%s000) - 60000))
auditquery="expand=true&endTime=${next_midnight}&startTime=${sixtySecondsAgo}"
MYCURL -X GET ${auditurl}?${auditquery}
if [[ ${CURL_RC} -eq 200 ]]; then
    printf "the audit query succeeded\n"
    cat ${CURL_OUT}
else
    printf "the audit query failed\n"
    cat ${CURL_OUT}
fi


CleanUp
exit 0

